import Control.Exception
import Options.Applicative
import Data.Semigroup ((<>))

import Types
import Control.Monad
import Unif
import Parser(parseTermFromString)


data Options = Options
              { firstTerm      :: String
              , secondTerm      :: String
              }

options :: Parser Options
options = Options
      <$> strOption
      (long "firstTerm"
       <> metavar "FIRSTTERM"
       <> value ""
       <> help "Le premier terme à unifier" )
      <*> strOption
      (long "secondTerm"
       <> metavar "SECONDTERM"
       <> value ""
       <> help "Le deuxième terme à unifier" )


main :: IO ()
main = runWithOptions 
  =<< execParser opts
  where
    opts = info (options <**> helper)
      ( fullDesc
     <> progDesc "Exécute l'algorithme d'unification sur deux termes"
     <> header "Header ... a completer" )

runWithOptions :: Options -> IO ()
runWithOptions (Options str1 str2) =
  (
  do
    str1'<- if(null str1) then promptAndGet "First term: " else return str1
    str2'<- if(null str2) then promptAndGet "Second term: " else return str2
    term1 <- myTransformer $ parseTermFromString str1' 
    term2 <- myTransformer $ parseTermFromString str2'
    result <- myTransformer $ unify [(term1,term2)]
    putStrLn $ "\nMGU: "++  show result
  )
  `catchAny`  printError


catchAny :: IO a -> (SomeException -> IO a) -> IO a
catchAny = Control.Exception.catch

printError :: SomeException -> IO ()
printError e = do
        putStrLn $ "Erreur : " ++ show e

promptAndGet :: String -> IO(String)
promptAndGet prompt = do
  putStr prompt >> getLine


myTransformer :: Show a => Either a b -> IO b
myTransformer action = case action of 
  Right x -> return x
  Left err -> fail $ show err
  
    
{-
main2 :: IO ()
main2 = do
  putStr "First term: "
  str1 <- getLine
  term1 <- myTransformer $ parseTermFromString str1 
  putStr "Second term: "
  str2 <- getLine
  term2 <- myTransformer $ parseTermFromString str2
  result <- myTransformer $ unify [(term1,term2)]
  putStrLn $ show result

-}
